#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.


from pydantic import BaseModel


class Article(BaseModel):
    """Base model for articles with full text.

    Attributes:
        source: the name of the source,, e.g. BBC.
        section: a tag to give the article, e.g. a category.
        url: the url of the original article.
        text: the text which represents the article.
    """
    source: str
    section: str
    url: str
    text: str



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from zennews.steps.sources.bbc import bbc_news_source, BBCParameters
from zennews.steps.report.report import post_summaries
from zennews.steps.summarize.bart_large_cnn_samsum import bart_large_cnn_samsum

from zennews.materializers.article_materializer import ArticleMaterializer  # noqa

SOURCE_STEP_MAPPING = {
    "bbc": {"step": bbc_news_source, "parameters": BBCParameters}
}


__all__ = [
    "post_summaries",
    "bbc_news_source",
    "bart_large_cnn_samsum",
    "SOURCE_STEP_MAPPING"
]




#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from typing import Type

from zenml.alerter.base_alerter import (
    BaseAlerterConfig,
    BaseAlerterFlavor,
    BaseAlerter,
)


class DiscordAlerterConfig(BaseAlerterConfig):
    """Configuration for the discord webhook alerter.

    Attributes:
        webhook_url_secret: str, the name of the secret which holds the
            url of the webhook.
    """
    webhook_url_secret: str


class DiscordAlerterFlavor(BaseAlerterFlavor):
    """Discord Alerter flavor."""

    @property
    def name(self):
        """Name of the discord flavor.

        Returns:
            the name of the discord flavor.
        """
        return "discord-webhook"

    @property
    def config_class(self) -> Type[BaseAlerterConfig]:
        """The configuration class for the discord alerter.

        Returns:
            The config class.
        """
        return DiscordAlerterConfig

    @property
    def implementation_class(self) -> Type[BaseAlerter]:
        """The implementation class for the discord alerter.

        Returns:
            The alerter class.
        """
        from zennews.alerter.discord_alerter import DiscordAlerter

        return DiscordAlerter



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.
from typing import List

import bbc_feeds
import requests
from bs4 import BeautifulSoup
from zenml.steps import step, BaseParameters

from zennews.models.article import Article

EXCLUDED_CLASS_IDENTIFIERS = ["PromoHeadline"]


class BBCParameters(BaseParameters):
    """Parameters to modify your feed from BBC news."""
    news_top_stories: bool = True
    news_uk: bool = False
    news_tech: bool = False
    news_business: bool = False
    news_entertainment: bool = False
    news_north_america: bool = False
    news_science: bool = False
    news_world: bool = False

    sports_all: bool = False
    sports_golf: bool = False
    sports_tennis: bool = False
    sports_boxing: bool = False
    sports_athletics: bool = False
    sports_cricket: bool = False
    sports_cycling: bool = False
    sports_formula1: bool = False
    sports_soccer: bool = False
    sports_swimming: bool = False

    weather: bool = True
    weather_city_id: int = 2867714  # TODO: Add description

    limit_per_section: int = 5

    def gather_feed(self):
        feed = {"news": {}, "sports": {}}
        for k, v in self.dict().items():
            if k.startswith("news_") or k.startswith("sports_"):
                if v:
                    section, subsection = k.split("_", 1)
                    section_class = getattr(bbc_feeds, section)
                    section_obj = section_class()
                    subsection_method = getattr(section_obj, subsection)
                    feed[section][subsection] = subsection_method(
                        limit=self.limit_per_section
                    )
        return feed


@step
def bbc_news_source(params: BBCParameters) -> List[Article]:
    """Step to download and parse news articles from BBC News."""
    articles: List[Article] = []

    # Derive the feed from the given params
    feed = params.gather_feed()

    # The source include "news" and "sports"
    for source, sections in feed.items():
        # The sections include all the categories in the params such as "news_X"
        for section_name, section_feed in sections.items():
            # Each section features its own RSS feed
            for story in section_feed:
                # Download the content from the story
                content = requests.get(story.link).content

                # Parse out all the paragraphs
                content = BeautifulSoup(content, 'html.parser')
                content_body = content.findAll('body')
                all_paragraphs = content_body[0].findAll('p')

                # Find out the longest paragraph in the content
                # TODO: Improve the scraping algorithm
                paragraphs = {}
                for p in all_paragraphs:
                    if p.get("class", None):
                        class_identifier = "-".join(p['class'])
                        if any(
                            [h not in class_identifier for h in
                             EXCLUDED_CLASS_IDENTIFIERS]
                            ):
                            if class_identifier not in paragraphs:
                                paragraphs[class_identifier] = {
                                    "len": 0,
                                    "text": ""
                                }

                            paragraphs[class_identifier]["len"] += len(p.text)
                            paragraphs[class_identifier]["text"] += p.text

                article_text = paragraphs[
                    max(
                        paragraphs,
                        key=lambda x: paragraphs[x]["len"]
                    )]["text"]

                # Create an "Article" from the result and add it to the list
                articles.append(
                    Article(
                        source="bbc",
                        section=f"{source}_{section_name}",
                        url=str(story.link),
                        text=article_text,
                    )
                )

    return articles



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from typing import List

from zenml.steps import step

from zennews.models import Article


@step
def bart_large_cnn_samsum(articles: List[Article]) -> List[Article]:
    """Step that generates summaries of the given list of news articles."""
    from transformers import AutoTokenizer, BartForConditionalGeneration

    model = BartForConditionalGeneration.from_pretrained(
        "facebook/bart-large-cnn"
    )
    tokenizer = AutoTokenizer.from_pretrained(
        "facebook/bart-large-cnn"
    )

    summarizations = []
    for a in articles:
        inputs = tokenizer(
            [a.text],
            max_length=1024,
            truncation=True,
            return_tensors="pt"
        )

        # Generate Summary
        summary_ids = model.generate(
            inputs["input_ids"],
            num_beams=2,
            min_length=0,
            max_length=256,
        )

        summary = tokenizer.batch_decode(
            summary_ids,
            skip_special_tokens=True,
            clean_up_tokenization_spaces=False
        )[0]

        summarizations.append(
            Article(
                source=a.source,
                section=a.section,
                url=a.url,
                text=summary,
            )
        )

    return summarizations



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.
import logging
from typing import List

from zenml.client import Client
from zenml.steps import step

from zennews.models import Article

from mdutils import MdUtils
from datetime import datetime


def generate_final_report(articles: List[Article]):
    md_file = MdUtils(file_name='report', title='ZenNews Summaries')
    md_file.new_header(
        title=f'From {articles[0].source.upper()} generated at '
              f'{datetime.now().strftime("%m/%d/%Y %H:%M:%S")}',
        level=1,
    )

    for a in articles:
        md_file.new_paragraph(f"**[{a.section}]** {a.text} [Link]({a.url})")

    return md_file.file_data_text


@step
def post_summaries(articles: List[Article]) -> str:
    """Step that reports the summaries through an alerter (if registered)"""
    final_report = generate_final_report(articles=articles)

    # Fetch the alerter if defined and use it to send the final report
    client = Client()
    if client.active_stack.alerter:
        client.active_stack.alerter.post(message=articles, params=None) # noqa
    else:
        logging.warning(
            'There is no alerter defined in your stack. The result will still'
            'be saved as an artifact in your artifact store.'
        )

    return final_report



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from typing import Optional, cast, Any

from discord import SyncWebhook
from zenml.alerter import BaseAlerter
from zenml.client import Client
from zenml.steps import BaseParameters

from zennews.alerter.discord_alerter_flavor import DiscordAlerterConfig


class DiscordAlerter(BaseAlerter):
    """Send messages to a Discord channel."""

    @property
    def config(self) -> DiscordAlerterConfig:
        """Get the config of this artifact store.
        Returns:
            The config of this artifact store.
        """
        return cast(DiscordAlerterConfig, self._config)

    def _get_webhook(self) -> SyncWebhook:
        """Create and return the discord webhook.

        Returns:
            the discord webhook object.
        """

        secret_manager = Client().active_stack.secrets_manager

        webhook_url = secret_manager.get_secret(
            self.config.webhook_url_secret
        ).content["WEBHOOK_URL"]

        return SyncWebhook.from_url(webhook_url)

    def post(
        self, message: Any, params: Optional[BaseParameters]
    ) -> bool:
        """Post a message to a Discord channel using a webhook.

        Args:
            message: str, message to be posted.
            params: parameters passed to the step (unused - will be deprecated).

        Returns:
            True if operation succeeded, else False.
        """

        webhook = self._get_webhook()

        # For the context of this project, the following implementation of the
        # discord alerter is very custom tailored to ZenNews. In order to
        # utilize this alerter in other projects, you would have to generalize
        # and change the implementation.

        from datetime import datetime
        from discord import Embed
        content = f'**From {message[0].source.upper()} generated ' \
                  f'at {datetime.now().strftime("%m/%d/%Y %H:%M:%S")}**'

        embeds = []
        for article in message:
            embeds.append(
                Embed(
                    description=article.text,
                    color=10181046,
                    url=article.url,
                    title=f"**{article.section}**",
                )
            )
        webhook.send(content=content, embeds=embeds)

        return True

    def ask(
        self, message: Any, params: Optional[BaseParameters]
    ) -> bool:
        """Post a message to a Discord channel and wait for approval.

        Args:
            message: str, initial message to be posted.
            params: parameters passed to the step (unused - will be deprecated).

        Returns:
            True if a user approved the operation, else False.
        """
        raise NotImplementedError(
            "The ask function of this alerter is not yet implemented."
        )



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from datetime import datetime
from typing import Dict, Any

import click
from zenml.client import Client
from zenml.enums import StackComponentType

from zennews.cli.base import cli
from zennews.cli.constants import SUPPORTED_ORCHESTRATORS
from zennews.cli.utils import (
    warning,
    error,
    stack_handler,
    title,
    build_pipeline,
    display_summaries,
    parse_schedule,
)
from zennews.steps import SOURCE_STEP_MAPPING


def generate_single_source_command(
    source_name: str, source_dict: Dict[Any, Any]
):
    """Function to generate dynamic Click commands for the ZenNews CLI."""

    source_step = source_dict['step']
    source_params = source_dict['parameters']

    @click.command(
        name=source_name,
        help=f'Run or schedule ZenNews pipelines for articles '
             f'from {source_name.upper()}.'
    )
    @click.option(
        '--schedule',
        type=click.Choice(
            ['debug', 'hourly', 'daily', 'weekly'], case_sensitive=False
        ),
        default=None,
        help="The amount of minutes to set as the frequency while scheduling."
    )
    @click.option(
        '--stack',
        type=str,
        default=None,
        help="The stack to use when scheduling the pipeline."
    )
    @click.option(
        '--force',
        '-f',
        is_flag=True,
        default=False,
        help="Flag to skip the confirmation."
    )
    def source_command(
        schedule: str,
        stack: str,
        force: bool,
        **kwargs: Dict[Any, Any],
    ) -> None:
        """Runs a ZenNews pipeline to summarize news articles.

        Args:
            schedule: str, the frequency to use if/when scheduling this
                pipeline.
            stack: str, the name of the stack to schedule the pipeline with.
            force: bool, flag to skip the confirmation if there is no
                schedule.
        """
        if schedule:
            # If there is a schedule, figure out which stack to use
            client = Client()
            stack_name = stack or client.active_stack_model.name

            with stack_handler(stack_name):
                active_stack = client.active_stack_model
                active_components = active_stack.components
                orchestrator = active_components[
                    StackComponentType.ORCHESTRATOR
                ][0]

                if orchestrator.flavor not in SUPPORTED_ORCHESTRATORS:
                    error('Does not work.')

                if StackComponentType.ALERTER not in active_components:
                    warning('There is no alerter!')

                pipeline = build_pipeline(
                    source_step=source_step,
                    source_params=source_params,
                    **kwargs
                )

                run_name = f'news_{source_name}_{{date}}_{{time}}'

                schedule_obj = parse_schedule(schedule, orchestrator.flavor)

                pipeline.run(run_name=run_name, schedule=schedule_obj)

        else:
            # If there is no schedule, RUN the pipeline locally
            if not force:
                warning(
                    'This will change your active ZenML stack to the default '
                    'stack and run the pipeline locally. This also means that '
                    'the pipeline will download and utilize the model locally.'
                )
                if not click.confirm('Would you like to continue?'):
                    error('Stopped the process.')

            # Build and run the pipeline
            title("Building and running the pipeline")

            with stack_handler("default"):
                pipeline = build_pipeline(
                    source_step=source_step,
                    source_params=source_params,
                    **kwargs
                )

                run_name = f'test_{source_name}' \
                           f'{datetime.now().strftime("%m_%d_%Y_%H_%M_%S")}'
                pipeline.run(run_name=run_name)

            # Get the run view of the pipeline and showcase the results
            title("pipeline results")

            from zenml.post_execution import get_run

            run_view = get_run(run_name)
            step_view = run_view.get_step("report")
            artifact_view = step_view.outputs['output']
            summaries = artifact_view.read()

            display_summaries(summaries)

    # Extract and add the step parameters to the Click command
    properties = source_params.schema()['properties']
    for property_name, property_values in properties.items():

        property_type = property_values.get('type', None)
        property_default = property_values.get('default', None)
        property_description = property_values.get('description', '')

        if property_type == 'boolean':
            option = click.option(
                f"--{property_name}",
                is_flag=True,
                default=property_default,
                help=property_description,
            )
        else:
            option = click.option(
                f"--{property_name}",
                type=str,
                default=property_default,
                help=property_description,
            )

        # Add it
        option(source_command)

    cli.add_command(source_command)


def generate_all_source_commands():
    """Function to generate CLI commands for all defined news sources."""
    # Iterate over all the sources available/implemented
    for s_name, s_dict in SOURCE_STEP_MAPPING.items():
        # Create all the subcommands
        generate_single_source_command(s_name, s_dict)


generate_all_source_commands()



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.
from zenml.config import DockerSettings
from zenml.pipelines import pipeline
from zennews import __version__
docker_settings = DockerSettings(requirements=[f"zennews={__version__}"])


@pipeline(enable_cache=False, settings={"docker": docker_settings})
def zen_news_pipeline(collect, summarize, report) -> None:
    """Defines an inference pipeline that summarizes a set of collected news.

    Args:
        collect: step which collects news from a specified source.
        summarize: step that uses an ML model to summarize the gathered news.
        report: step responsible for generating and sending a report from the
            provided summaries.
    """
    news = collect()
    summaries = summarize(news)
    report(summaries)



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.
import os

import click

from zennews.cli.constants import APP_NAME


@click.group(APP_NAME)
def cli():
    """CLI base command for ZenML."""

    # Supress warning messages during client initializations
    os.environ["ZENML_ENABLE_REPO_INIT_WARNINGS"] = "false"

    click.secho(
        r"""
                 ______          _   _                   
                |___  /         | \ | |                  
                   / / ___ _ __ |  \| | _____      _____ 
                  / / / _ \ '_ \| . ` |/ _ \ \ /\ / / __|
                 / /_|  __/ | | | |\  |  __/\ V  V /\__ \
                /_____\___|_| |_|_| \_|\___| \_/\_/ |___/
                                                                               
                      This is where you get the news.
         """,
        fg="magenta",
    )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['zennews',
 'zennews.alerter',
 'zennews.cli',
 'zennews.materializers',
 'zennews.models',
 'zennews.pipelines',
 'zennews.steps',
 'zennews.steps.report',
 'zennews.steps.sources',
 'zennews.steps.summarize']

package_data = \
{'': ['*']}

install_requires = \
['bbc-feeds==2.1',
 'discord.py==2.1.0',
 'mdutils==1.4.0',
 'torch==1.13.1',
 'transformers==4.26.0',
 'zenml[server]==0.32.1']

entry_points = \
{'console_scripts': ['zennews = zennews.cli.base:cli']}

setup_kwargs = {
    'name': 'zennews',
    'version': '0.1.4',
    'description': 'ZenNews: Get summarized news on a schedule.',
    'long_description': "# ZenNews: Generate summarized news on a schedule\n\nIn today's information age, we are bombarded with a constant stream of news \nand media from a variety of sources. Summarizing tasks, particularly when it \ncomes to news sources, can be a powerful tool for efficient consumption of \ninformation. They distill complex or lengthy content into easily \ndigestible chunks that can be scanned and absorbed quickly, allowing us to \nkeep up with the news without being overwhelmed. They can also help us separate \nthe signal from the noise, highlighting the most important details and helping \nus identify what's worth further investigation. \n\nThis is where **ZenNews** come into the play. It offers a tool which can \nautomate the summarization process and save users time and effort while \nproviding them with the information they need. This can be particularly valuable \nfor busy professionals or anyone who wants to keep up with the news but doesn't \nhave the time to read every article in full.\n\n# The goal of the project\n\nThe definition of the concrete use-case aside, this project aims to showcase \nsome of the advantages that ZenML brings to the table. Some major points we \nwould like to highlight include:\n\n- **The ease of use**: ZenML features a simple and clean Python SDK. As you can \nobserve in this example, it is not only used to define your steps and \npipelines but also to access/manage the resources and artifacts that you \ninteract with along the way. This makes it significantly easier for our users \nto build their applications around ZenML.\n\n- **The extensibility**: ZenML is an extendable framework. ML projects often \nrequire custom-tailored solutions and what you get out-of-the-box may not be \nwhat you need. This is why ZenML is using base abstractions to allow you \nto create your own solutions without reinventing the whole wheel. You can find \ngreat examples of this feature by taking a look at the custom materializer \n(ArticleMaterializer) and the custom stack component (DiscordAlerter) \nimplemented within the context of this project.\n\n- **Stack vs Code**: One of the main features of ZenML is rooted within the \nconcept of our stacks. As you follow this example, you will see that there is a \ncomplete separation between the code and the infrastructure that the pipeline \nis running on. In fact, by utilizing just one flag, it is possible to switch \nfrom a local default stack to a remote deployment with scheduled pipelines.\n\n- **The scalability**: This is a small PoC-like example which aims to prove \nthat ZenML can help you streamline your workflows and accelerate your \ndevelopment process. However, it barely scratches the surface of how you can \nimprove it even further. For more information, check this section.\n\n# Base installation\n\nThe **ZenNews** project is designed as a \n[PyPI package](https://pypi.org/project/zennews/)\nthat you can install it through `pip`:\n\n```bash\npip install zennews\n```\n\nThe package comes equipped with the following set of **key** pieces:\n\n- **The pipeline**: The `zen_news_pipeline` is the main pipeline in this \nworkflow. In total, it features three steps, namely `collect`, \n`summarize` and `report`. The first step is responsible for collecting \narticles, the second step summarizes them, and the last step creates a \nreport and posts it.\n- **The steps**: There is a concrete implementation for each step defined above.\n  - For the `collect` step, we have the `bbc_news_source` which (on default) \n  collects the top stories off of the BBC news feed and prepares `Article` \n  objects. \n  - For the `summarize` step, we have implemented `bart_large_cnn_samsum`\n  step. As the name suggests, it uses bart model to generate summaries. \n  - Ultimately, for the `report` step, we have implemented the `post_summaries` \n  step. It showcases how a generalized step can function within a ZenML \n  pipeline and uses an alerter to share the results.\n- **The materializer**: As mentioned above, the steps within our pipeline are \nusing the concept of `Article`s to define their input and output space. Through\nthis, we can show how to handle the materialization of these artifacts when it \ncomes a data type that is not a built-in.\n- **The custom stack component**: The ultimate goal of an application such as \n`ZenNews` is to display to outcomes to the user directly. With this project, \nwe have used this as a chance to show the extensibility of ZenML in terms of the\nstack components and implemented a `DiscordAlerter`.\n- **The CLI application**: The example also includes a \n- [Click](https://click.palletsprojects.com/en/8.1.x/) CLI application. \nIt utilizes how easily you can use our Python SDK to build your application \naround your ZenML workflows. In order to see it action simply execute:\n\n  ```bash\n  zennews --help \n   ```\n\n# Test it locally right away\n\nOnce you installed the `zennews` package, you are ready to test it out locally \nright away. The following command will get the top five articles from the BBC\nnews feed, summarize them and present the results to you. \n\n> Warning: This will temporarily switch your active ZenML stack to the \n> **default** stack and when the pipeline runs, you will download the model \n> to your local machine.\n\n```bash\nzennews bbc\n```\n\nYou can also parameterize this process. In order to see the possible \nparameters, please use:\n\n```bash\nzennews bbc --help\n```\n\n# Switching to scheduled pipelines with Vertex\n\nThe potential of an application like `ZenNews` can be only unlocked by \nscheduling summarization pipelines instead of manually triggering them \nyourself. In order to showcase it, we will set up a fully remote GCP stack \nand use the `VertexOrchestrator` to schedule the pipeline.\n\n## Deploy ZenML on GCP\n\nBefore you start building the stack, you need to deploy ZenML on GCP. For more \ninformation on how you can achieve do that, please check \n[the corresponding docs page](https://docs.zenml.io/getting-started/deploying-zenml).\n\n## ZenNews Stack\n\nOnce the ZenML is deployed, we can start to build up our stack. Our stack will \nconsist of the following components:\n\n- [GCP Secrets Manager](https://docs.zenml.io/component-gallery/secrets-managers/gcp)\n- [GCP Container Registry](https://docs.zenml.io/component-gallery/container-registries/gcloud)\n- [GCS Artifact Store](https://docs.zenml.io/component-gallery/artifact-stores/gcloud-gcs)\n- [Vertex Orchestrator](https://docs.zenml.io/component-gallery/orchestrators/gcloud-vertexai)\n- [Discord Alerter (part of the `zennews` package)](src/zennews/alerter/discord_alerter.py)\n \nLet's start by installing the `gcp` integration:\n\n```bash\nzenml integration install gcp\n```\n\n### Secrets Manager\n\nThe first component to register is a\n[GCP secrets manager](https://docs.zenml.io/component-gallery/secrets-managers/gcp). \nThe command is quite straightforward. You just have to give it a name and \nprovide the ID of your project on GCP.\n\n```bash\nzenml secrets-manager register <SECRETS_MANAGER_NAME> \\\n    --flavor=gcp \\\n    --project_id=<PROJECT_ID>\n```\n\n### Container Registry\n\nThe second component is a \n[GCP container registry](https://docs.zenml.io/component-gallery/container-registries/gcloud). \nSimilar to the previous component, you just need to provide a name and the \nURI to your container registry on GCP.\n\n```bash\nzenml container-registry register <CONTAINER_REGISTERY_NAME> \\\n    --flavor=gcp \\\n    --uri=<REGISTRY_URI>\n```\n\n### Artifact Store\n\nThe next component on the list is a \n[GCS artifact store](https://docs.zenml.io/component-gallery/artifact-stores/gcloud-gcs). \nIn order to register it, all you have to do is to provide the path to your GCS\nbucket:\n\n```bash \nzenml artifact-store register <ARTIFACT_STORE_NAME> \\\n    --flavor=gcp \\\n    --path=<PATH_TO_BUCKET> \n```\n\n### Orchestrator\n\nFollowing the artifact store, we will register a\n[Vertex AI orchestrator.](https://docs.zenml.io/component-gallery/orchestrators/gcloud-vertexai)\n\n```bash\nzenml orchestrator register <ORCHESTRATOR_NAME> \\\n    --flavor=vertex \\\n    --project=<PROJECT_ID> \\\n    --location=<GCP_LOCATION>\n```\n\nYou need to simply provide the id of your project and the name of your GCP \nregion.\n\n### GCP Stack\n\nWith these four components, we are ready to establish and activate the base \nversion of our GCP stack.\n\n```bash\nzenml stack register <STACK_NAME> \\\n    -x <SECRETS_MANAGER_NAME> \\\n    -c <CONTAINER_REGISTERY_NAME> \\\n    -a <ARTIFACT_STORE_NAME> \\\n    -o <ORCHESTRATOR_NAME> \\\n    --set\n```\n\n### Alerter \n\nThe last component in our stack is a special case. As mentioned before\nthe `zennews` package already comes equipped with a custom stack component \nimplementation, namely the `DiscordAlerter`. In a nutshell, it uses the \n[**discord.py**](https://discordpy.readthedocs.io/en/stable/index.html) package\nto send messages via a webhook to a discord text channel. You can find the \nimplementation right [here](src/zennews/alerter/discord_alerter.py).\n\nThe following sections show how we can register `DiscordAlerter` as a custom \nflavor , create an instance of it and update our stack.\n\n#### Registering the custom flavor\n\n All you \nhave to do to register such custom flavor is to provide the corresponding\nsource path to the flavor class.\n\n```bash\nzenml alerter flavor register zennews.alerter.discord_alerter_flavor.DiscordAlerterFlavor\n```\n\nZenML will import and add that to the list of available alerter flavors.\n\n```bash \nzenml alerter flavor list\n```\n\n#### Registering the alerter\n\nNow that the flavor is registered, you can create an alerter with the flavor\n`discord-webhook`. Through this example, you will also see how you can use \nsecret references to handle sensitive information during the registration of \nstack components.\n\nLet's start by registering the secret:\n\n```bash\nzenml secrets-manager secret register <SECRET_NAME> \\\n    --webhook_url=<ACTUAL_URL_OF_THE_WEBHOOK>\n```\n\nThis will use the secrets manager in our active GCP stack. Once the secret \nregistration is complete, you can register your alerter as follows:\n\n```bash\nzenml alerter register <ALERTER_NAME> \\\n    --flavor discord-webhook \\\n    --webhook_url=<SECRET_REFERENCE>  # formatted as {{SECRET_NAME:WEBHOOK_URL}}\n```\n\n#### Updating the stack\n\nThe last step is to update our stack with our new alerter:\n\n```bash\nzenml stack update <STACK_NAME> -al <ALERTER_NAME>\n```\n\n## Scheduling pipelines through the `zennews` CLI\n\nNow the stack is set up, you can use the `--schedule` option when you run your \n`zennews` pipeline. There are three possible values that you can use for the \n`schedule` option: `hourly`, `daily` (every day at 9 AM) or `weekly` (every\nMonday at 9 AM).\n\n```bash\nzennews bbc --schedule daily\n```\n\nThis will use your active stack (the GCP stack) and schedule your ZenNews \npipeline.\n\n## Limitations, future improvements and upcoming changes\n\nBefore we end this project, it is also important to talk about the limitations\nwe faced, the possible future improvements, and changes which are already in \nmotion:\n\n- The first limitation of the ZenNews is the number of supported news sources.\nAs this project was initially designed as a PoC, the only supported news \nsource is BBC. However, thanks to our design, it is really easy to expand this \nlist by adding additional steps, which consume data and create `Article` \nobjects.\n- The ability to schedule pipelines through ZenML played a critical role \nwithin the context of this project. However, this feature has its own \nlimitations. While you can create scheduled pipelines, once the pipeline and \nits schedule is created, you can not cancel or modify the behaviour of this \nscheduled pipeline. This means, if you want to cancel it, you have to do it \nover the orchestrator yourself.\n- The other limitation regarding the schedules is the format. As of now, the \nCLI application takes the user input and converts it into a cron expression.\nAny orchestrator which does not support these expressions will not applicable.\n- As the ZenML team, we have been looking for ways to improve the interface \nof our base alerters. You might see some changes in the upcoming releases. \n- Similar to the alerters, we are working on improving the management of our \nsecrets. \n \nTune in to [our slack](https://zenml.io/slack-invite/) to stay updated about \nthe upcoming changes and ask any questions you might have.\n",
    'author': 'ZenML GmbH',
    'author_email': 'info@zenml.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://zenml.io/project/zen-news-summarization',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<3.11',
}


setup(**setup_kwargs)



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.
from typing import List, Optional, Any

import click
from zenml.client import Client
from zenml.config.schedule import Schedule

from zennews.models import Article


def error(text: str) -> None:
    """Wrapper around the click.ClickException.

    Args:
        text, str, the exception text.

    Raises:
        click.ClickException with the defined style.
    """
    raise click.ClickException(message=click.style(text, fg="red", bold=True))


def warning(text: str) -> None:
    """Wrapper around the 'click.echo' for warning messages.

    Args:
        text, str, the warning message.
    """
    click.secho(text, fg='yellow', bold=True)


def title(text: str) -> None:
    click.secho(f"\n ----- {text.upper()} ----- \n", fg='cyan', bold=True)


class stack_handler(object):
    """Context manager that switches the active stack temporarily."""

    def __init__(self, target_stack_name: str = 'default') -> None:
        """Initialization of the stack handler.

        Args:
            target_stack_name: str, the name of the target stack
        """
        self.active_stack_name = None
        self.target_stack_name = target_stack_name

    def __enter__(self) -> "stack_handler":
        """Enter function of the stack handler.

        Saves the name of the current active stack and activates the temporary
        target stack.

        Returns:
            the handler instance.
        """
        client = Client()

        self.active_stack_name = client.active_stack_model.name

        if self.active_stack_name != self.target_stack_name:
            warning(
                "Temporarily changing the active stack from "
                f"{self.active_stack_name} to {self.target_stack_name}!"
            )

        client.activate_stack(self.target_stack_name)
        return self

    def __exit__(
        self,
        type_: Optional[Any],
        value: Optional[Any],
        traceback: Optional[Any],
    ) -> Any:
        """Exit function of the stack handler.

        Sets the previous stack as the active stack again.

        Args:
            type_: The class of the exception
            value: The instance of the exception
            traceback: The traceback of the exception
        """

        client = Client()

        if self.active_stack_name != self.target_stack_name:
            warning(
                "Changing the active stack back from "
                f"{self.active_stack_name} to {self.target_stack_name}!"
            )

        client.activate_stack(self.active_stack_name)


def build_pipeline(source_step, source_params, **kwargs):
    """Create an instance of a zennews pipeline with the given steps.

    Args:
        source_step: the step class for the source
        source_params: the parameters class for the source
        kwargs: the parameters

    Returns:
        an instance of a zennews pipeline
    """

    from zennews.pipelines import zen_news_pipeline
    from zennews.steps import bart_large_cnn_samsum, post_summaries

    pipeline = zen_news_pipeline(
        collect=source_step(source_params.parse_obj(kwargs)),
        summarize=bart_large_cnn_samsum(),
        report=post_summaries(),
    )

    return pipeline


def parse_schedule(frequency: str, flavor: str) -> Schedule:
    """Create a schedule object that can be used by the orchestrator flavor.

    Args:
        frequency: str, the frequency set for that profile
        flavor: str, the flavor of the orchestrator

    Returns:
         the proper Schedule object
    """
    if flavor == "vertex":
        if frequency == 'debug':
            return Schedule(cron_expression="*/5 * * * *")
        elif frequency == 'hourly':
            return Schedule(cron_expression="0 * * * *")
        elif frequency == 'daily':
            return Schedule(cron_expression="0 9 * * *")
        elif frequency == 'weekly':
            return Schedule(cron_expression="0 9 * * MON")
        else:
            raise ValueError('Please use one of the supported values.')

    else:
        raise NotImplementedError(
            'The schedule parser can only be used by the Vertex orchestrator!'
        )


def display_summaries(summaries: str) -> None:
    """Display the articles on the CLI."""
    from rich.markdown import Markdown
    from rich.console import Console

    md = Markdown(summaries)
    Console().print(md)



#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

import json
import os
from typing import Type

from zenml.enums import ArtifactType
from zenml.io import fileio
from zenml.materializers.base_materializer import BaseMaterializer

from zennews.models import Article


class ArticleMaterializer(BaseMaterializer):
    """Custom materializer implementation for articles."""
    ASSOCIATED_TYPES = (Article,)
    ASSOCIATED_ARTIFACT_TYPE = ArtifactType.DATA

    def load(self, data_type: Type[Article]) -> Article:
        """Read an article from the artifact store"""
        super().load(data_type)
        with fileio.open(os.path.join(self.uri, 'article.json'), 'r') as f:
            return Article.parse_raw(json.load(f))

    def save(self, my_obj: Article) -> None:
        """Write an article to artifact store"""
        super().save(my_obj)
        with fileio.open(os.path.join(self.uri, 'article.json'), 'w') as f:
            json.dump(my_obj.json(), f)
